@@ -1,6 +1,6 @@
 /*
-     This file is part of libmicrohttpd
-     (C) 2007, 2008 Daniel Pittman and Christian Grothoff
+    This file is part of libmicrohttpd
+     (C) 2007, 2008, 2009, 2010 Daniel Pittman and Christian Grothoff
 
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Lesser General Public
@@ -333,6 +333,14 @@ try_ready_normal_body (struct MHD_Connection *connection)
        (response->data_size + response->data_start >
 	connection->response_write_position) )
     return MHD_YES; /* response already ready */
+#if LINUX
+  if ( (response->fd != -1) &&
+       (0 == (connection->daemon->options & MHD_USE_SSL)) )
+    {
+      /* will use sendfile, no need to bother response crc */
+      return MHD_YES; 
+    }
+#endif
   ret = response->crc (response->crc_cls,
                        connection->response_write_position,
                        response->data,
@@ -450,7 +458,9 @@ try_ready_chunked_body (struct MHD_Connection *connection)
     }
   if (ret > 0xFFFFFF)
     ret = 0xFFFFFF;
-  SPRINTF (cbuf, "%X\r\n", ret);
+  snprintf (cbuf, 
+	    sizeof (cbuf),
+	    "%X\r\n", ret);
   cblen = strlen (cbuf);
   EXTRA_CHECK (cblen <= sizeof (cbuf));
   memcpy (&connection->write_buffer[sizeof (cbuf) - cblen], cbuf, cblen);
@@ -577,13 +587,28 @@ build_header_response (struct MHD_Connection *connection)
   enum MHD_ValueKind kind;
   const char *reason_phrase;
 
+  EXTRA_CHECK (NULL != connection->version);
+  if (0 == strlen(connection->version))
+    {
+      data = MHD_pool_allocate (connection->pool, 0, MHD_YES);
+      connection->write_buffer = data;
+      connection->write_buffer_append_offset = 0;
+      connection->write_buffer_send_offset = 0;
+      connection->write_buffer_size = 0;
+      return MHD_YES;
+    }
   if (connection->state == MHD_CONNECTION_FOOTERS_RECEIVED)
     {
       add_extra_headers (connection);
       reason_phrase = MHD_get_reason_phrase_for (connection->responseCode);
       SPRINTF (code,
                "%s %u %s\r\n",
-               MHD_HTTP_VERSION_1_1, connection->responseCode, reason_phrase);
+               (0 == strcasecmp (MHD_HTTP_VERSION_1_0,
+				 connection->version)) 
+	       ? MHD_HTTP_VERSION_1_0 
+	       : MHD_HTTP_VERSION_1_1,
+	       connection->responseCode, 
+	       reason_phrase);
       off = strlen (code);
       /* estimate size */
       size = off + 2;           /* extra \r\n at the end */
@@ -657,7 +682,12 @@ transmit_error_response (struct MHD_Connection *connection,
 {
   struct MHD_Response *response;
 
-  /* die, header far too long to be reasonable */
+  if (connection->version == NULL)
+    {
+      /* we were unable to process the full header line, so we don't
+	 really know what version the client speaks; assume 1.0 */
+      connection->version = MHD_HTTP_VERSION_1_0;
+    }
   connection->state = MHD_CONNECTION_FOOTERS_RECEIVED;
   connection->read_closed = MHD_YES;
 #if HAVE_MESSAGES
@@ -693,7 +723,8 @@ static void
 do_fd_set (int fd, fd_set * set, int *max_fd)
 {
   FD_SET (fd, set);
-  if (fd > *max_fd)
+  if ( (NULL != max_fd) &&
+       (fd > *max_fd) )
     *max_fd = fd;
 }
 
@@ -755,12 +786,17 @@ MHD_connection_get_pollfd(struct MHD_Connection *connection, struct MHD_Pollfd *
 #endif
       switch (connection->state)
         {
+#if HTTPS_SUPPORT     
+	case MHD_TLS_CONNECTION_INIT:
+	  if (0 == gnutls_record_get_direction (connection->tls_session))
+            p->events |= MHD_POLL_ACTION_IN;
+	  else
+	    p->events |= MHD_POLL_ACTION_OUT;
+	  break;
+#endif
         case MHD_CONNECTION_INIT:
         case MHD_CONNECTION_URL_RECEIVED:
         case MHD_CONNECTION_HEADER_PART_RECEIVED:
-#if HTTPS_SUPPORT
-        case MHD_TLS_CONNECTION_INIT:
-#endif
           /* while reading headers, we always grow the
              read buffer if needed, no size-check required */
           if ((connection->read_closed) &&
@@ -1561,6 +1597,7 @@ parse_connection_headers (struct MHD_Connection *connection)
   const char *clen;
   unsigned long long cval;
   struct MHD_Response *response;
+  const char *enc;
 
   parse_cookie_header (connection);
   if ((0 != (MHD_USE_PEDANTIC_CHECKS & connection->daemon->options))
@@ -1605,21 +1642,19 @@ parse_connection_headers (struct MHD_Connection *connection)
     }
   else
     {
-      if (NULL == MHD_lookup_connection_value (connection,
-                                               MHD_HEADER_KIND,
-                                               MHD_HTTP_HEADER_TRANSFER_ENCODING))
+      enc = MHD_lookup_connection_value (connection,
+					 MHD_HEADER_KIND,
+					 MHD_HTTP_HEADER_TRANSFER_ENCODING);
+      if (NULL == enc)
         {
-          /* this request does not have a body */
+          /* this request (better) not have a body */
           connection->remaining_upload_size = 0;
         }
       else
         {
           connection->remaining_upload_size = MHD_SIZE_UNKNOWN;
-          if (0 ==
-              strcasecmp (MHD_lookup_connection_value
-                          (connection, MHD_HEADER_KIND,
-                           MHD_HTTP_HEADER_TRANSFER_ENCODING), "chunked"))
-            connection->have_chunked_upload = MHD_YES;
+          if (0 == strcasecmp (enc, "chunked"))
+	    connection->have_chunked_upload = MHD_YES;
         }
     }
 }
@@ -1768,29 +1803,13 @@ MHD_connection_handle_write (struct MHD_Connection *connection)
               connection->state = MHD_CONNECTION_NORMAL_BODY_UNREADY;
               break;
             }
-#if HTTPS_SUPPORT
-          if (connection->daemon->options & MHD_USE_SSL)
-            {
-              ret = MHD__gnutls_record_send (connection->tls_session,
-                                             &connection->response->data
-                                             [connection->
-                                              response_write_position -
-                                              response->data_start],
-                                             response->data_size -
-                                             (connection->response_write_position
-                                              - response->data_start));
-            }
-          else
-#endif
-            {
-              ret = connection->send_cls (connection,
-                                          &response->data
-                                          [connection->response_write_position
-                                           - response->data_start],
-                                          response->data_size -
-                                          (connection->response_write_position
-                                           - response->data_start));
-            }
+	  ret = connection->send_cls (connection,
+				      &response->data
+				      [connection->response_write_position
+				       - response->data_start],
+				      response->data_size -
+				      (connection->response_write_position
+				       - response->data_start));
 #if DEBUG_SEND_DATA
           if (ret > 0)
             FPRINTF (stderr,
@@ -1844,8 +1863,6 @@ MHD_connection_handle_write (struct MHD_Connection *connection)
             connection_close_error (connection);
           return MHD_NO;
         case MHD_TLS_CONNECTION_INIT:
-        case MHD_TLS_HELLO_REQUEST:
-        case MHD_TLS_HANDSHAKE_FAILED:
           EXTRA_CHECK (0);
           break;
         default:
@@ -2223,10 +2240,6 @@ MHD_set_http_calbacks (struct MHD_Connection *connection)
   connection->idle_handler = &MHD_connection_handle_idle;
 }
 
-#if HTTPS_SUPPORT
-#include "gnutls_int.h"
-#include "gnutls_record.h"
-#endif
 
 /**
  * Obtain information about the given connection.
@@ -2247,13 +2260,17 @@ MHD_get_connection_info (struct MHD_Connection *connection,
     case MHD_CONNECTION_INFO_CIPHER_ALGO:
       if (connection->tls_session == NULL)
 	return NULL;
-      return (const union MHD_ConnectionInfo *) &connection->
-        tls_session->security_parameters.read_bulk_cipher_algorithm;
+      connection->cipher = gnutls_cipher_get (connection->tls_session);
+      return (const union MHD_ConnectionInfo *) &connection->cipher;
     case MHD_CONNECTION_INFO_PROTOCOL:
       if (connection->tls_session == NULL)
 	return NULL;
-      return (const union MHD_ConnectionInfo *) &connection->
-        tls_session->security_parameters.version;
+      connection->protocol = gnutls_protocol_get_version (connection->tls_session);
+      return (const union MHD_ConnectionInfo *) &connection->protocol;
+    case MHD_CONNECTION_INFO_GNUTLS_SESSION:
+      if (connection->tls_session == NULL)
+	return NULL;
+      return (const union MHD_ConnectionInfo *) &connection->tls_session;
 #endif
     case MHD_CONNECTION_INFO_CLIENT_ADDRESS:
       return (const union MHD_ConnectionInfo *) &connection->addr;
