@@ -1,6 +1,6 @@
 /*
   This file is part of libmicrohttpd
-  (C) 2007, 2008, 2009 Daniel Pittman and Christian Grothoff
+  (C) 2007, 2008, 2009, 2010 Daniel Pittman and Christian Grothoff
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -32,14 +32,18 @@
 
 #if HTTPS_SUPPORT
 #include "connection_https.h"
-#include "gnutls_int.h"
-#include "gnutls_global.h"
+#include <gnutls/gnutls.h>
+#include <gcrypt.h>
 #endif
 
 #ifdef HAVE_POLL_H
 #include <poll.h>
 #endif
 
+#ifdef LINUX
+#include <sys/sendfile.h>
+#endif
+
 /**
  * Default connection limit.
  */
@@ -331,25 +335,7 @@ MHD_ip_limit_del(struct MHD_Daemon *daemon,
 }
 
 #if HTTPS_SUPPORT
-pthread_mutex_t MHD_gnutls_init_mutex;
-
-/**
- * Note: code duplication with code in MHD_gnutls_priority.c
- *
- * @return 0
- */
-static int
-_set_priority (MHD_gtls_priority_st * st, const int *list)
-{
-  int num = 0;
-
-  while ((list[num] != 0) && (num < MAX_ALGOS))
-    num++;
-  st->num_algorithms = num;
-  memcpy (st->priority, list, num * sizeof (int));
-  return 0;
-}
-
+static pthread_mutex_t MHD_gnutls_init_mutex;
 
 /**
  * Callback for receiving data from the socket.
@@ -362,7 +348,7 @@ _set_priority (MHD_gtls_priority_st * st, const int *list)
 static ssize_t
 recv_tls_adapter (struct MHD_Connection *connection, void *other, size_t i)
 {
-  return MHD__gnutls_record_recv (connection->tls_session, other, i);
+  return gnutls_record_recv (connection->tls_session, other, i);
 }
 
 /**
@@ -377,7 +363,7 @@ static ssize_t
 send_tls_adapter (struct MHD_Connection *connection,
                   const void *other, size_t i)
 {
-  return MHD__gnutls_record_send (connection->tls_session, other, i);
+  return gnutls_record_send (connection->tls_session, other, i);
 }
 
 
@@ -389,8 +375,8 @@ send_tls_adapter (struct MHD_Connection *connection,
 static int
 MHD_init_daemon_certificate (struct MHD_Daemon *daemon)
 {
-  MHD_gnutls_datum_t key;
-  MHD_gnutls_datum_t cert;
+  gnutls_datum_t key;
+  gnutls_datum_t cert;
 
   /* certificate & key loaded from memory */
   if (daemon->https_mem_cert && daemon->https_mem_key)
@@ -400,9 +386,9 @@ MHD_init_daemon_certificate (struct MHD_Daemon *daemon)
       cert.data = (unsigned char *) daemon->https_mem_cert;
       cert.size = strlen (daemon->https_mem_cert);
 
-      return MHD__gnutls_certificate_set_x509_key_mem (daemon->x509_cred,
-                                                       &cert, &key,
-                                                       GNUTLS_X509_FMT_PEM);
+      return gnutls_certificate_set_x509_key_mem (daemon->x509_cred,
+						  &cert, &key,
+						  GNUTLS_X509_FMT_PEM);
     }
 #if HAVE_MESSAGES
   MHD_DLOG (daemon, "You need to specify a certificate and key location\n");
@@ -420,9 +406,9 @@ MHD_TLS_init (struct MHD_Daemon *daemon)
 {
   switch (daemon->cred_type)
     {
-    case MHD_GNUTLS_CRD_CERTIFICATE:
+    case GNUTLS_CRD_CERTIFICATE:
       if (0 !=
-          MHD__gnutls_certificate_allocate_credentials (&daemon->x509_cred))
+          gnutls_certificate_allocate_credentials (&daemon->x509_cred))
         return GNUTLS_E_MEMORY_ERROR;
       return MHD_init_daemon_certificate (daemon);
     default:
@@ -439,6 +425,12 @@ MHD_TLS_init (struct MHD_Daemon *daemon)
 /**
  * Obtain the select sets for this daemon.
  *
+ * @param daemon daemon to get sets from
+ * @param read_fd_set read set
+ * @param write_fd_set write set
+ * @param except_fd_set except set
+ * @param max_fd increased to largest FD added (if larger
+ *               than existing value); can be NULL
  * @return MHD_YES on success, MHD_NO if this
  *         daemon was not started with the right
  *         options for this call.
@@ -460,7 +452,7 @@ MHD_get_fdset (struct MHD_Daemon *daemon,
 
   FD_SET (fd, read_fd_set);
   /* update max file descriptor */
-  if ((*max_fd) < fd)
+  if ((*max_fd) < fd) 
     *max_fd = fd;
 
   con_itr = daemon->connections;
@@ -615,8 +607,7 @@ recv_param_adapter (struct MHD_Connection *connection, void *other, size_t i)
     return -1;
   if (0 != (connection->daemon->options & MHD_USE_SSL))
     return RECV (connection->socket_fd, other, i, MSG_NOSIGNAL);
-  else
-    return RECV (connection->socket_fd, other, i, MSG_NOSIGNAL | MSG_DONTWAIT);
+  return RECV (connection->socket_fd, other, i, MSG_NOSIGNAL | MSG_DONTWAIT);
 }
 
 /**
@@ -631,14 +622,73 @@ static ssize_t
 send_param_adapter (struct MHD_Connection *connection,
                     const void *other, size_t i)
 {
+#if LINUX
+  int fd;
+  off_t offset;
+  int ret;
+#endif
   if (connection->socket_fd == -1)
     return -1;
   if (0 != (connection->daemon->options & MHD_USE_SSL))
     return SEND (connection->socket_fd, other, i, MSG_NOSIGNAL);
-  else
-    return SEND (connection->socket_fd, other, i, MSG_NOSIGNAL | MSG_DONTWAIT);
+#if LINUX
+  if ( (connection->write_buffer_append_offset ==
+	connection->write_buffer_send_offset) &&
+       (NULL != connection->response) &&
+       (-1 != (fd = connection->response->fd)) )
+    {
+      /* can use sendfile */
+      offset = (off_t) connection->response_write_position;
+      ret = sendfile (connection->socket_fd, 
+		      fd,
+		      &offset,
+		      connection->response->total_size - offset);
+      if ( (ret == -1) &&
+	   (errno == EINTR) )
+	return 0;
+      return ret;
+    }
+#endif
+  return SEND (connection->socket_fd, other, i, MSG_NOSIGNAL | MSG_DONTWAIT);
 }
 
+
+#if HTTPS_SUPPORT
+/**
+ * Set if a socket should use non-blocking IO.
+ * @param fd socket
+ */
+static void
+socket_set_nonblocking (int fd)
+{
+#if MINGW
+  u_long mode;
+  mode = 1;
+  if (ioctlsocket (fd, FIONBIO, &mode) == SOCKET_ERROR)
+    {
+      SetErrnoFromWinsockError (WSAGetLastError ());
+#if HAVE_MESSAGES
+      FPRINTF(stderr, "Failed to make socket non-blocking: %s\n", 
+	      STRERROR (errno));
+#endif
+    }
+#else
+
+  /* not MINGW */
+  int flags = fcntl (fd, F_GETFL);
+  if ( (flags == -1) ||
+       (0 != fcntl (fd, F_SETFL, flags | O_NONBLOCK)) )
+    {
+#if HAVE_MESSAGES
+      FPRINTF(stderr, "Failed to make socket non-blocking: %s\n", 
+	      STRERROR (errno));
+#endif
+    }
+#endif
+}
+#endif
+
+
 /**
  * Accept an incoming connection and create the MHD_Connection object for
  * it.  This function also enforces policy by way of checking with the
@@ -776,17 +826,22 @@ MHD_accept_connection (struct MHD_Daemon *daemon)
       connection->recv_cls = &recv_tls_adapter;
       connection->send_cls = &send_tls_adapter;
       connection->state = MHD_TLS_CONNECTION_INIT;
-      MHD_set_https_calbacks (connection);
-      MHD__gnutls_init (&connection->tls_session, GNUTLS_SERVER);
-      MHD__gnutls_priority_set (connection->tls_session,
-                                connection->daemon->priority_cache);
+      MHD_set_https_callbacks (connection);
+      gnutls_init (&connection->tls_session, GNUTLS_SERVER);
+      gnutls_priority_set (connection->tls_session,
+			   daemon->priority_cache);
+      if (0 == (daemon->options & MHD_USE_THREAD_PER_CONNECTION))
+	{
+	  /* use non-blocking IO for gnutls */
+	  socket_set_nonblocking (connection->socket_fd);
+	}
       switch (connection->daemon->cred_type)
         {
           /* set needed credentials for certificate authentication. */
-        case MHD_GNUTLS_CRD_CERTIFICATE:
-          MHD__gnutls_credentials_set (connection->tls_session,
-                                       MHD_GNUTLS_CRD_CERTIFICATE,
-                                       connection->daemon->x509_cred);
+        case GNUTLS_CRD_CERTIFICATE:
+          gnutls_credentials_set (connection->tls_session,
+				  GNUTLS_CRD_CERTIFICATE,
+				  connection->daemon->x509_cred);
           break;
         default:
 #if HAVE_MESSAGES
@@ -808,13 +863,13 @@ MHD_accept_connection (struct MHD_Daemon *daemon)
 		     );
  	  return MHD_NO;
         }
-      MHD__gnutls_transport_set_ptr (connection->tls_session,
-                                     (MHD_gnutls_transport_ptr_t) connection);
-      MHD__gnutls_transport_set_pull_function (connection->tls_session,
-                                               (MHD_gtls_pull_func) &
+      gnutls_transport_set_ptr (connection->tls_session,
+				(gnutls_transport_ptr_t) connection);
+      gnutls_transport_set_pull_function (connection->tls_session,
+					  (gnutls_pull_func) &
                                                recv_param_adapter);
-      MHD__gnutls_transport_set_push_function (connection->tls_session,
-                                               (MHD_gtls_push_func) &
+      gnutls_transport_set_push_function (connection->tls_session,
+					  (gnutls_push_func) &
                                                send_param_adapter);
     }
 #endif
@@ -885,7 +940,7 @@ MHD_cleanup_connections (struct MHD_Daemon *daemon)
           MHD_pool_destroy (pos->pool);
 #if HTTPS_SUPPORT
           if (pos->tls_session != NULL)
-            MHD__gnutls_deinit (pos->tls_session);
+            gnutls_deinit (pos->tls_session);
 #endif
           MHD_ip_limit_del (daemon, (struct sockaddr*)pos->addr, pos->addr_len);
           free (pos->addr);
@@ -908,6 +963,7 @@ MHD_cleanup_connections (struct MHD_Daemon *daemon)
  * returned value is how long select should at most
  * block, not the timeout value set for connections.
  *
+ * @param daemon daemon to query for timeout
  * @param timeout set to the timeout (in milliseconds)
  * @return MHD_YES on success, MHD_NO if timeouts are
  *        not used (or no connections exist that would
@@ -1213,6 +1269,10 @@ parse_options_va (struct MHD_Daemon *daemon,
   enum MHD_OPTION opt;
   struct MHD_OptionItem *oa;
   unsigned int i;
+#if HTTPS_SUPPORT
+  int ret;
+  const char *pstr;
+#endif
   
   while (MHD_OPTION_END != (opt = va_arg (ap, enum MHD_OPTION)))
     {
@@ -1245,19 +1305,16 @@ parse_options_va (struct MHD_Daemon *daemon,
           break;
         case MHD_OPTION_THREAD_POOL_SIZE:
           daemon->worker_pool_size = va_arg (ap, unsigned int);
+	  if (daemon->worker_pool_size >= SIZE_MAX / sizeof (struct MHD_Daemon))
+	    {
+#if HAVE_MESSAGES
+	      FPRINTF (stderr,
+		       "Specified thread pool size too big\n");
+#endif
+	      return MHD_NO;
+	    }
           break;
 #if HTTPS_SUPPORT
-        case MHD_OPTION_PROTOCOL_VERSION:
-	  if (daemon->options & MHD_USE_SSL)
-	    _set_priority (&daemon->priority_cache->protocol,
-			   va_arg (ap, const int *));
-#if HAVE_MESSAGES
-	  else
-	    FPRINTF (stderr,
-		     "MHD HTTPS option %d passed to MHD but MHD_USE_SSL not set\n",
-		     opt);	  
-#endif
-          break;
         case MHD_OPTION_HTTPS_MEM_KEY:
 	  if (daemon->options & MHD_USE_SSL)
 	    daemon->https_mem_key = va_arg (ap, const char *);
@@ -1278,18 +1335,27 @@ parse_options_va (struct MHD_Daemon *daemon,
 		     opt);	  
 #endif
           break;
-        case MHD_OPTION_CIPHER_ALGORITHM:
-	  if (daemon->options & MHD_USE_SSL)
-	    _set_priority (&daemon->priority_cache->cipher,
-			   va_arg (ap, const int *));
+	case MHD_OPTION_HTTPS_CRED_TYPE:
+	  daemon->cred_type = va_arg (ap, gnutls_credentials_type_t);
+	  break;
+        case MHD_OPTION_HTTPS_PRIORITIES:
+	  ret = gnutls_priority_init (&daemon->priority_cache,
+				      pstr = va_arg (ap, const char*),
+				      NULL);
 #if HAVE_MESSAGES
-	  else
+	  if (ret != GNUTLS_E_SUCCESS)
 	    FPRINTF (stderr,
-		     "MHD HTTPS option %d passed to MHD but MHD_USE_SSL not set\n",
-		     opt);	  
+		     "Setting priorities to `%s' failed: %s\n",
+		     pstr,
+		     gnutls_strerror (ret));
 #endif	  
+	  if (ret != GNUTLS_E_SUCCESS)
+	    return MHD_NO;
           break;
 #endif
+	case MHD_OPTION_LISTEN_SOCKET:
+	  daemon->socket_fd = va_arg (ap, int);	  
+	  break;
         case MHD_OPTION_EXTERNAL_LOGGER:
 #if HAVE_MESSAGES
           daemon->custom_error_log =
@@ -1329,7 +1395,8 @@ parse_options_va (struct MHD_Daemon *daemon,
 		    return MHD_NO;
 		  break;
 		  /* all options taking 'int' or 'enum' */
-		case MHD_OPTION_CRED_TYPE:
+		case MHD_OPTION_HTTPS_CRED_TYPE:
+		case MHD_OPTION_LISTEN_SOCKET:
 		  if (MHD_YES != parse_options (daemon,
 						servaddr,
 						opt,
@@ -1341,8 +1408,7 @@ parse_options_va (struct MHD_Daemon *daemon,
 		case MHD_OPTION_SOCK_ADDR:
 		case MHD_OPTION_HTTPS_MEM_KEY:
 		case MHD_OPTION_HTTPS_MEM_CERT:
-		case MHD_OPTION_PROTOCOL_VERSION:
-		case MHD_OPTION_CIPHER_ALGORITHM:
+		case MHD_OPTION_HTTPS_PRIORITIES:
 		case MHD_OPTION_ARRAY:
 		  if (MHD_YES != parse_options (daemon,
 						servaddr,
@@ -1373,7 +1439,7 @@ parse_options_va (struct MHD_Daemon *daemon,
         default:
 #if HAVE_MESSAGES
           if ((opt >= MHD_OPTION_HTTPS_MEM_KEY) &&
-              (opt <= MHD_OPTION_CIPHER_ALGORITHM))
+              (opt <= MHD_OPTION_HTTPS_PRIORITIES))
             {
               FPRINTF (stderr,
                        "MHD HTTPS option %d passed to MHD compiled without HTTPS support\n",
@@ -1429,6 +1495,12 @@ MHD_start_daemon_va (unsigned int options,
   if (retVal == NULL)
     return NULL;
   memset (retVal, 0, sizeof (struct MHD_Daemon));
+#if HTTPS_SUPPORT
+  gnutls_priority_init (&retVal->priority_cache,
+			"NORMAL",
+			NULL);
+#endif
+  retVal->socket_fd = -1;
   retVal->options = (enum MHD_OPTION)options;
   retVal->port = port;
   retVal->apc = apc;
@@ -1461,9 +1533,7 @@ MHD_start_daemon_va (unsigned int options,
 #endif
 	  mhd_panic (mhd_panic_cls, __FILE__, __LINE__, NULL);
 	}
-      /* set default priorities */
-      MHD_tls_set_default_priority (&retVal->priority_cache, "", NULL);
-      retVal->cred_type = MHD_GNUTLS_CRD_CERTIFICATE;
+      retVal->cred_type = GNUTLS_CRD_CERTIFICATE;
     }
 #endif
 
@@ -1507,28 +1577,103 @@ MHD_start_daemon_va (unsigned int options,
       return NULL;
     }
 #endif
-  if ((options & MHD_USE_IPv6) != 0)
+  if (retVal->socket_fd == -1)
+    {
+      if ((options & MHD_USE_IPv6) != 0)
 #if HAVE_INET6
-    socket_fd = SOCKET (PF_INET6, SOCK_STREAM, 0);
+	socket_fd = SOCKET (PF_INET6, SOCK_STREAM, 0);
 #else
-    {
+      {
 #if HAVE_MESSAGES
-      fprintf (stderr, "AF_INET6 not supported\n");
+	fprintf (stderr, "AF_INET6 not supported\n");
 #endif
-      free (retVal);
-      return NULL;
+	free (retVal);
+	return NULL;
+      }
+#endif
+      else
+	socket_fd = SOCKET (PF_INET, SOCK_STREAM, 0);
+      if (socket_fd == -1)
+	{
+#if HAVE_MESSAGES
+	  if ((options & MHD_USE_DEBUG) != 0)
+	    FPRINTF (stderr, "Call to socket failed: %s\n", STRERROR (errno));
+#endif
+	  free (retVal);
+	  return NULL;
+	}
+      if ((SETSOCKOPT (socket_fd,
+		       SOL_SOCKET,
+		       SO_REUSEADDR,
+		       &on, sizeof (on)) < 0) && ((options & MHD_USE_DEBUG) != 0))
+	{
+#if HAVE_MESSAGES
+	  FPRINTF (stderr, "setsockopt failed: %s\n", STRERROR (errno));
+#endif
+	}
+      
+      /* check for user supplied sockaddr */
+#if HAVE_INET6
+      if ((options & MHD_USE_IPv6) != 0)
+	addrlen = sizeof (struct sockaddr_in6);
+      else
+#endif
+	addrlen = sizeof (struct sockaddr_in);
+      if (NULL == servaddr)
+	{
+#if HAVE_INET6
+	  if ((options & MHD_USE_IPv6) != 0)
+	    {
+	      memset (&servaddr6, 0, sizeof (struct sockaddr_in6));
+	      servaddr6.sin6_family = AF_INET6;
+	      servaddr6.sin6_port = htons (port);
+	      servaddr = (struct sockaddr *) &servaddr6;
+	    }
+	  else
+#endif
+	    {
+	      memset (&servaddr4, 0, sizeof (struct sockaddr_in));
+	      servaddr4.sin_family = AF_INET;
+	      servaddr4.sin_port = htons (port);
+	      servaddr = (struct sockaddr *) &servaddr4;
+	    }
+	}
+      retVal->socket_fd = socket_fd;
+
+      if ((options & MHD_USE_IPv6) != 0)
+	{
+	  const int on = 1;
+	  setsockopt (socket_fd, 
+		      IPPROTO_IPV6, IPV6_V6ONLY, 
+		      &on, sizeof (on));      
+	}
+      if (BIND (socket_fd, servaddr, addrlen) == -1)
+	{
+#if HAVE_MESSAGES
+	  if ((options & MHD_USE_DEBUG) != 0)
+	    FPRINTF (stderr,
+		     "Failed to bind to port %u: %s\n", port, STRERROR (errno));
+#endif
+	  CLOSE (socket_fd);
+	  free (retVal);
+	  return NULL;
+	}
+      
+      if (LISTEN (socket_fd, 20) < 0)
+	{
+#if HAVE_MESSAGES
+	  if ((options & MHD_USE_DEBUG) != 0)
+	    FPRINTF (stderr,
+		     "Failed to listen for connections: %s\n", STRERROR (errno));
+#endif
+	  CLOSE (socket_fd);
+	  free (retVal);
+	  return NULL;
+	}      
     }
-#endif
   else
-    socket_fd = SOCKET (PF_INET, SOCK_STREAM, 0);
-  if (socket_fd == -1)
     {
-#if HAVE_MESSAGES
-      if ((options & MHD_USE_DEBUG) != 0)
-        FPRINTF (stderr, "Call to socket failed: %s\n", STRERROR (errno));
-#endif
-      free (retVal);
-      return NULL;
+      socket_fd = retVal->socket_fd;
     }
 #ifndef WINDOWS
   if ( (socket_fd >= FD_SETSIZE) &&
@@ -1546,66 +1691,6 @@ MHD_start_daemon_va (unsigned int options,
       return NULL;
     }
 #endif
-  if ((SETSOCKOPT (socket_fd,
-                   SOL_SOCKET,
-                   SO_REUSEADDR,
-                   &on, sizeof (on)) < 0) && (options & MHD_USE_DEBUG) != 0)
-    {
-#if HAVE_MESSAGES
-      FPRINTF (stderr, "setsockopt failed: %s\n", STRERROR (errno));
-#endif
-    }
-
-  /* check for user supplied sockaddr */
-#if HAVE_INET6
-  if ((options & MHD_USE_IPv6) != 0)
-    addrlen = sizeof (struct sockaddr_in6);
-  else
-#endif
-    addrlen = sizeof (struct sockaddr_in);
-  if (NULL == servaddr)
-    {
-#if HAVE_INET6
-      if ((options & MHD_USE_IPv6) != 0)
-        {
-          memset (&servaddr6, 0, sizeof (struct sockaddr_in6));
-          servaddr6.sin6_family = AF_INET6;
-          servaddr6.sin6_port = htons (port);
-          servaddr = (struct sockaddr *) &servaddr6;
-        }
-      else
-#endif
-        {
-          memset (&servaddr4, 0, sizeof (struct sockaddr_in));
-          servaddr4.sin_family = AF_INET;
-          servaddr4.sin_port = htons (port);
-          servaddr = (struct sockaddr *) &servaddr4;
-        }
-    }
-  retVal->socket_fd = socket_fd;
-  if (BIND (socket_fd, servaddr, addrlen) == -1)
-    {
-#if HAVE_MESSAGES
-      if ((options & MHD_USE_DEBUG) != 0)
-        FPRINTF (stderr,
-                 "Failed to bind to port %u: %s\n", port, STRERROR (errno));
-#endif
-      CLOSE (socket_fd);
-      free (retVal);
-      return NULL;
-    }
-
-  if (LISTEN (socket_fd, 20) < 0)
-    {
-#if HAVE_MESSAGES
-      if ((options & MHD_USE_DEBUG) != 0)
-        FPRINTF (stderr,
-                 "Failed to listen for connections: %s\n", STRERROR (errno));
-#endif
-      CLOSE (socket_fd);
-      free (retVal);
-      return NULL;
-    }
 
   if (0 != pthread_mutex_init (&retVal->per_ip_connection_mutex, NULL))
     {
@@ -1857,9 +1942,8 @@ MHD_stop_daemon (struct MHD_Daemon *daemon)
 #if HTTPS_SUPPORT
   if (daemon->options & MHD_USE_SSL)
     {
-      MHD__gnutls_priority_deinit (daemon->priority_cache);
       if (daemon->x509_cred)
-        MHD__gnutls_certificate_free_credentials (daemon->x509_cred);
+        gnutls_certificate_free_credentials (daemon->x509_cred);
       /* lock MHD_gnutls_global mutex since it uses reference counting */
       if (0 != pthread_mutex_lock (&MHD_gnutls_init_mutex))
 	{
@@ -1956,6 +2040,10 @@ sigalrmHandler (int sig)
 #define ATTRIBUTE_DESTRUCTOR
 #endif  // __GNUC__
 
+#if HTTPS_SUPPORT
+GCRY_THREAD_OPTION_PTHREAD_IMPL;
+#endif
+
 /**
  * Initialize the signal handler for SIGALRM
  * and do other setup work.
@@ -1976,7 +2064,8 @@ void ATTRIBUTE_CONSTRUCTOR MHD_init ()
   plibc_init ("GNU", "libmicrohttpd");
 #endif
 #if HTTPS_SUPPORT
-  MHD__gnutls_global_init ();
+  gcry_control (GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
+  gnutls_global_init ();
   if (0 != pthread_mutex_init(&MHD_gnutls_init_mutex, NULL))
     abort();
 #endif
@@ -1985,7 +2074,7 @@ void ATTRIBUTE_CONSTRUCTOR MHD_init ()
 void ATTRIBUTE_DESTRUCTOR MHD_fini ()
 {
 #if HTTPS_SUPPORT
-  MHD__gnutls_global_deinit ();
+  gnutls_global_deinit ();
   if (0 != pthread_mutex_destroy(&MHD_gnutls_init_mutex))
     mhd_panic (mhd_panic_cls, __FILE__, __LINE__, NULL);
 #endif
