@@ -1,21 +1,21 @@
 /*
- This file is part of libmicrohttpd
- (C) 2007 Daniel Pittman and Christian Grothoff
-
- This library is free software; you can redistribute it and/or
- modify it under the terms of the GNU Lesser General Public
- License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
-
- This library is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- Lesser General Public License for more details.
-
- You should have received a copy of the GNU Lesser General Public
- License along with this library; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- */
+  This file is part of libmicrohttpd
+  (C) 2007, 2008, 2009, 2010 Daniel Pittman and Christian Grothoff
+  
+  This library is free software; you can redistribute it and/or
+  modify it under the terms of the GNU Lesser General Public
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
+  
+  This library is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  Lesser General Public License for more details.
+  
+  You should have received a copy of the GNU Lesser General Public
+  License along with this library; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+*/
 
 /**
  * @file internal.h
@@ -30,7 +30,7 @@
 #include "platform.h"
 #include "microhttpd.h"
 #if HTTPS_SUPPORT
-#include "gnutls.h"
+#include <gnutls/gnutls.h>
 #endif
 
 #define EXTRA_CHECKS MHD_YES
@@ -186,16 +186,16 @@ struct MHD_Response
   pthread_mutex_t mutex;
 
   /**
-   * Reference count for this response.  Free
-   * once the counter hits zero.
-   */
-  unsigned int reference_count;
-
-  /**
-   * Set to -1 if size is not known.
+   * Set to MHD_SIZE_UNKNOWN if size is not known.
    */
   uint64_t total_size;
 
+  /**
+   * At what offset in the stream is the
+   * beginning of data located?
+   */
+  uint64_t data_start;
+
   /**
    * Size of data.
    */
@@ -207,10 +207,15 @@ struct MHD_Response
   size_t data_buffer_size;
 
   /**
-   * At what offset in the stream is the
-   * beginning of data located?
+   * Reference count for this response.  Free
+   * once the counter hits zero.
    */
-  uint64_t data_start;
+  unsigned int reference_count;
+
+  /**
+   * File-descriptor if this response is FD-backed.
+   */
+  int fd;
 
 };
 
@@ -345,20 +350,7 @@ enum MHD_CONNECTION_STATE
    * Handshake messages will be processed in this state & while
    * in the 'MHD_TLS_HELLO_REQUEST' state
    */
-  MHD_TLS_CONNECTION_INIT = MHD_CONNECTION_CLOSED + 1,
-
-  /**
-   * This state indicates the server has send a 'Hello Request' to
-   * the client & a renegotiation of the handshake is in progress.
-   *
-   * Handshake messages will processed in this state & while
-   * in the 'MHD_TLS_CONNECTION_INIT' state
-   */
-  MHD_TLS_HELLO_REQUEST,
-
-  MHD_TLS_HANDSHAKE_FAILED,
-
-  MHD_TLS_HANDSHAKE_COMPLETE
+  MHD_TLS_CONNECTION_INIT = MHD_CONNECTION_CLOSED + 1
 
 };
 
@@ -660,7 +652,17 @@ struct MHD_Connection
   /**
    * State required for HTTPS/SSL/TLS support.
    */
-  MHD_gtls_session_t tls_session;
+  gnutls_session_t tls_session;
+
+  /**
+   * Memory location to return for protocol session info.
+   */
+  int protocol;
+
+  /**
+   * Memory location to return for protocol session info.
+   */
+  int cipher;
 #endif
 };
 
@@ -795,26 +797,26 @@ struct MHD_Daemon
   unsigned short port;
 
 #if HTTPS_SUPPORT
+  /**
+   * Desired cipher algorithms.
+   */
+  gnutls_priority_t priority_cache;
+
   /**
    * What kind of credentials are we offering
    * for SSL/TLS?
    */
-  enum MHD_GNUTLS_CredentialsType cred_type;
+  gnutls_credentials_type_t cred_type;
 
   /**
    * Server x509 credentials
    */
-  MHD_gtls_cert_credentials_t x509_cred;
-
-  /**
-   * Cipher priority cache
-   */
-  MHD_gnutls_priority_t priority_cache;
+  gnutls_certificate_credentials_t x509_cred;
 
   /**
    * Diffie-Hellman parameters
    */
-  MHD_gtls_dh_params_t dh_params;
+  gnutls_dh_params_t dh_params;
 
   /**
    * Pointer to our SSL/TLS key (in ASCII) in memory.
@@ -825,6 +827,7 @@ struct MHD_Daemon
    * Pointer to our SSL/TLS certificate (in ASCII) in memory.
    */
   const char *https_mem_cert;
+
 #endif
 
   /**
