@@ -1,6 +1,6 @@
 /*
      This file is part of libmicrohttpd
-     (C) 2007, 2009 Daniel Pittman and Christian Grothoff
+     (C) 2007, 2009, 2010 Daniel Pittman and Christian Grothoff
 
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Lesser General Public
@@ -163,7 +163,7 @@ MHD_get_response_header (struct MHD_Response *response, const char *key)
  * Create a response object.  The response object can be extended with
  * header information and then be used any number of times.
  *
- * @param size size of the data portion of the response, -1 for unknown
+ * @param size size of the data portion of the response, MHD_SIZE_UNKNOWN for unknown
  * @param block_size preferred block size for querying crc (advisory only,
  *                   MHD may still call crc using smaller chunks); this
  *                   is essentially the buffer size used for IO, clients
@@ -189,6 +189,7 @@ MHD_create_response_from_callback (uint64_t size,
   if (retVal == NULL)
     return NULL;
   memset (retVal, 0, sizeof (struct MHD_Response));
+  retVal->fd = -1;
   retVal->data = (void *) &retVal[1];
   retVal->data_buffer_size = block_size;
   if (pthread_mutex_init (&retVal->mutex, NULL) != 0)
@@ -204,6 +205,72 @@ MHD_create_response_from_callback (uint64_t size,
   return retVal;
 }
 
+
+/**
+ * Given a file descriptor, read data from the file
+ * to generate the response.
+ * 
+ * @param cls pointer to the response
+ * @param pos offset in the file to access
+ * @param buf where to write the data
+ * @param max number of bytes to write at most
+ * @return number of bytes written
+ */
+static int
+file_reader (void *cls, uint64_t pos, char *buf, int max)
+{
+  struct MHD_Response *response = cls;
+  int ret;
+
+  pthread_mutex_lock (&response->mutex);
+  (void) lseek (response->fd, pos, SEEK_SET);
+  ret = read (response->fd, buf, max);
+  pthread_mutex_unlock (&response->mutex);
+  return ret;
+}
+
+
+/**
+ * Destroy file reader context.  Closes the file
+ * descriptor.
+ *
+ * @param cls pointer to file descriptor
+ */
+static void
+free_callback (void *cls)
+{
+  struct MHD_Response *response = cls;
+  (void) close (response->fd);
+  response->fd = -1;
+}
+
+
+/**
+ * Create a response object.  The response object can be extended with
+ * header information and then be used any number of times.
+ *
+ * @param size size of the data portion of the response
+ * @param fd file descriptor referring to a file on disk with the data
+ * @return NULL on error (i.e. invalid arguments, out of memory)
+ */
+struct MHD_Response *MHD_create_response_from_fd (size_t size,
+						  int fd)
+{
+  struct MHD_Response *ret;
+
+  ret = MHD_create_response_from_callback (size,
+					   4 * 1024,
+					   &file_reader,
+					   NULL,
+					   &free_callback);
+  if (ret == NULL)
+    return NULL;
+  ret->fd = fd;
+  ret->crc_cls = ret;
+  return ret;
+}
+
+
 /**
  * Create a response object.  The response object can be extended with
  * header information and then be used any number of times.
@@ -229,6 +296,7 @@ MHD_create_response_from_data (size_t size,
   if (retVal == NULL)
     return NULL;
   memset (retVal, 0, sizeof (struct MHD_Response));
+  retVal->fd = -1;
   if (pthread_mutex_init (&retVal->mutex, NULL) != 0)
     {
       free (retVal);
@@ -270,7 +338,7 @@ MHD_destroy_response (struct MHD_Response *response)
   if (response == NULL)
     return;
   pthread_mutex_lock (&response->mutex);
-  if (0 != --response->reference_count)
+  if (0 != --(response->reference_count))
     {
       pthread_mutex_unlock (&response->mutex);
       return;
@@ -295,7 +363,7 @@ void
 MHD_increment_response_rc (struct MHD_Response *response)
 {
   pthread_mutex_lock (&response->mutex);
-  response->reference_count++;
+  (response->reference_count)++;
   pthread_mutex_unlock (&response->mutex);
 }
 
