@@ -34,10 +34,17 @@ file_reader (void *cls, uint64_t pos, char *buf, int max)
 {
   FILE *file = cls;
 
-  fseek (file, pos, SEEK_SET);
+  (void)  fseek (file, pos, SEEK_SET);
   return fread (buf, 1, max, file);
 }
 
+static void
+free_callback (void *cls)
+{
+  FILE *file = cls;
+  fclose (file);
+}
+
 static int
 ahc_echo (void *cls,
           struct MHD_Connection *connection,
@@ -62,7 +69,10 @@ ahc_echo (void *cls,
       return MHD_YES;
     }
   *ptr = NULL;                  /* reset when done */
-  file = fopen (&url[1], "rb");
+  if (0 == stat (&url[1], &buf))
+    file = fopen (&url[1], "rb");
+  else
+    file = NULL;
   if (file == NULL)
     {
       response = MHD_create_response_from_data (strlen (PAGE),
@@ -73,12 +83,15 @@ ahc_echo (void *cls,
     }
   else
     {
-      stat (&url[1], &buf);
       response = MHD_create_response_from_callback (buf.st_size, 32 * 1024,     /* 32k page size */
                                                     &file_reader,
                                                     file,
-                                                    (MHD_ContentReaderFreeCallback)
-                                                    & fclose);
+                                                    &free_callback);
+      if (response == NULL)
+	{
+	  fclose (file);
+	  return MHD_NO;
+	}
       ret = MHD_queue_response (connection, MHD_HTTP_OK, response);
       MHD_destroy_response (response);
     }
@@ -90,9 +103,9 @@ main (int argc, char *const *argv)
 {
   struct MHD_Daemon *d;
 
-  if (argc != 3)
+  if (argc != 2)
     {
-      printf ("%s PORT SECONDS-TO-RUN\n", argv[0]);
+      printf ("%s PORT\n", argv[0]);
       return 1;
     }
   d = MHD_start_daemon (MHD_USE_THREAD_PER_CONNECTION | MHD_USE_DEBUG,
@@ -100,7 +113,7 @@ main (int argc, char *const *argv)
                         NULL, NULL, &ahc_echo, PAGE, MHD_OPTION_END);
   if (d == NULL)
     return 1;
-  sleep (atoi (argv[2]));
+  (void) getc (stdin);
   MHD_stop_daemon (d);
   return 0;
 }
