@@ -35,8 +35,7 @@
 #include "platform.h"
 #include <microhttpd.h>
 #include <sys/stat.h>
-
-#include "gnutls.h"
+#include <gnutls/gnutls.h>
 #include <gcrypt.h>
 
 #define BUF_SIZE 1024
@@ -101,10 +100,17 @@ file_reader (void *cls, uint64_t pos, char *buf, int max)
 {
   FILE *file = cls;
 
-  fseek (file, pos, SEEK_SET);
+  (void) fseek (file, pos, SEEK_SET);
   return fread (buf, 1, max, file);
 }
 
+static void
+file_free_callback (void *cls)
+{
+  FILE *file = cls;
+  fclose (file);
+}
+
 /* HTTP access handler call back */
 static int
 http_ahc (void *cls,
@@ -131,7 +137,11 @@ http_ahc (void *cls,
     }
   *ptr = NULL;                  /* reset when done */
 
-  file = fopen (&url[1], "rb");
+  if ( (0 == stat (url, &buf)) &&
+       (S_ISREG (buf.st_mode)) )
+    file = fopen (&url[1], "rb");
+  else
+    file = NULL;
   if (file == NULL)
     {
       response = MHD_create_response_from_data (strlen (EMPTY_PAGE),
@@ -142,11 +152,14 @@ http_ahc (void *cls,
     }
   else
     {
-      stat (url, &buf);
       response = MHD_create_response_from_callback (buf.st_size, 32 * 1024,     /* 32k PAGE_NOT_FOUND size */
                                                     &file_reader, file,
-                                                    (MHD_ContentReaderFreeCallback)
-                                                    & fclose);
+                                                    &file_free_callback);
+      if (response == NULL)
+	{
+	  fclose (file);
+	  return MHD_NO;
+	}
       ret = MHD_queue_response (connection, MHD_HTTP_OK, response);
       MHD_destroy_response (response);
     }
@@ -158,7 +171,7 @@ main (int argc, char *const *argv)
 {
   struct MHD_Daemon *TLS_daemon;
 
-  if (argc == 3)
+  if (argc == 2)
     {
       /* TODO check if this is truly necessary -  disallow usage of the blocking /dev/random */
       /* gcry_control(GCRYCTL_ENABLE_QUICK_RANDOM, 0); */
@@ -172,7 +185,7 @@ main (int argc, char *const *argv)
     }
   else
     {
-      printf ("Usage: %s HTTP-PORT SECONDS-TO-RUN\n", argv[0]);
+      printf ("Usage: %s HTTP-PORT\n", argv[0]);
       return 1;
     }
 
@@ -186,7 +199,7 @@ main (int argc, char *const *argv)
       printf ("MHD daemon listening on port %d\n", atoi (argv[1]));
     }
 
-  sleep (atoi (argv[2]));
+  (void) getc (stdin);
 
   MHD_stop_daemon (TLS_daemon);
 
