@@ -1,6 +1,6 @@
 /*
      This file is part of libmicrohttpd
-     (C) 2006, 2007, 2008, 2009 Christian Grothoff (and other contributing authors)
+     (C) 2006, 2007, 2008, 2009, 2010 Christian Grothoff (and other contributing authors)
 
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Lesser General Public
@@ -41,10 +41,10 @@
  * continue" are understood and handled automatically).<p>
  *
  * MHD understands POST data and is able to decode certain formats
- * (at the moment only "application/x-www-form-urlencoded") if the
- * entire data fits into the allowed amount of memory for the
- * connection.  Unsupported encodings and large POST submissions are
- * provided as a stream to the main application (and thus can be
+ * (at the moment only "application/x-www-form-urlencoded" and
+ * "mulitpart/formdata"). Unsupported encodings and large POST 
+ * submissions may require the application to manually process
+ * the stream, which is provided to the main application (and thus can be
  * processed, just not conveniently by MHD).<p>
  *
  * The header file defines various constants used by the HTTP protocol.
@@ -81,7 +81,7 @@ extern "C"
 /**
  * Current version of the library.
  */
-#define MHD_VERSION 0x00040600
+#define MHD_VERSION 0x00090000
 
 /**
  * MHD-internal return code for "YES".
@@ -97,7 +97,11 @@ extern "C"
  * Constant used to indicate unknown size (use when
  * creating a response).
  */
+#ifdef UINT64_MAX
+#define MHD_SIZE_UNKNOWN UINT64_MAX
+#else
 #define MHD_SIZE_UNKNOWN  ((uint64_t) -1LL)
+#endif
 
 /**
  * HTTP response codes.
@@ -408,28 +412,24 @@ enum MHD_OPTION
 
   /**
    * Daemon credentials type.
-   * This option should be followed by one of the values listed in
-   * "enum MHD_GNUTLS_CredentialsType".
+   * Followed by an argument of type
+   * "gnutls_credentials_type_t".
    */
-  MHD_OPTION_CRED_TYPE = 10,
+  MHD_OPTION_HTTPS_CRED_TYPE = 10,
 
   /**
-   * SSL/TLS protocol version.
-   *
-   * Memory pointer to a zero (MHD_GNUTLS_PROTOCOL_END) terminated
-   * (const) array of 'enum MHD_GNUTLS_Protocol' values representing the
-   * protocol versions to this server should support. Unsupported
-   * requests will be droped by the server.
+   * Memory pointer to a "const char*" specifying the
+   * cipher algorithm (default: "NORMAL").
    */
-  MHD_OPTION_PROTOCOL_VERSION = 11,
+  MHD_OPTION_HTTPS_PRIORITIES = 11,
 
   /**
-   * Memory pointer to a zero (MHD_GNUTLS_CIPHER_UNKNOWN)
-   * terminated (const) array of 'enum MHD_GNUTLS_CipherAlgorithm'
-   * representing the cipher priority order to which the HTTPS
-   * daemon should adhere.
+   * Pass a listen socket for MHD to use (systemd-style).  If this
+   * option is used, MHD will not open its own listen socket(s). The
+   * argument passed must be of type "int" and refer to an
+   * existing socket that has been bound to a port and is listening.
    */
-  MHD_OPTION_CIPHER_ALGORITHM = 12,
+  MHD_OPTION_LISTEN_SOCKET = 12,
 
   /**
    * Use the given function for logging error messages.
@@ -583,35 +583,6 @@ enum MHD_RequestTerminationCode
 
 };
 
-/**
- * List of symmetric ciphers.
- * Note that not all listed algorithms are necessarily
- * supported by all builds of MHD.
- */
-enum MHD_GNUTLS_CipherAlgorithm
-{
-  MHD_GNUTLS_CIPHER_UNKNOWN = 0,
-  MHD_GNUTLS_CIPHER_NULL = 1,
-  MHD_GNUTLS_CIPHER_ARCFOUR_128,
-  MHD_GNUTLS_CIPHER_3DES_CBC,
-  MHD_GNUTLS_CIPHER_AES_128_CBC,
-  MHD_GNUTLS_CIPHER_AES_256_CBC
-};
-
-/**
- * SSL/TLS Protocol types.
- * Note that not all listed algorithms are necessarily
- * supported by all builds of MHD.
- */
-enum MHD_GNUTLS_Protocol
-{
-  MHD_GNUTLS_PROTOCOL_END = 0,
-  MHD_GNUTLS_PROTOCOL_SSL3 = 1,
-  MHD_GNUTLS_PROTOCOL_TLS1_0,
-  MHD_GNUTLS_PROTOCOL_TLS1_1,
-  MHD_GNUTLS_PROTOCOL_TLS1_2,
-  MHD_GNUTLS_PROTOCOL_VERSION_UNKNOWN = 0xff
-};
 
 /**
  * Values of this enum are used to specify what
@@ -635,7 +606,12 @@ enum MHD_ConnectionInfoType
    * Obtain IP address of the client.
    * Takes no extra arguments.
    */
-  MHD_CONNECTION_INFO_CLIENT_ADDRESS
+  MHD_CONNECTION_INFO_CLIENT_ADDRESS,
+
+  /**
+   * Get the GNUTLS session handle.
+   */
+  MHD_CONNECTION_INFO_GNUTLS_SESSION
 };
 
 /**
@@ -930,7 +906,7 @@ void MHD_stop_daemon (struct MHD_Daemon *daemon);
  * @param write_fd_set write set
  * @param except_fd_set except set
  * @param max_fd increased to largest FD added (if larger
- *               than existing value)
+ *               than existing value); can be NULL
  * @return MHD_YES on success, MHD_NO if this
  *         daemon was not started with the right
  *         options for this call.
@@ -1071,7 +1047,7 @@ MHD_queue_response (struct MHD_Connection *connection,
  * Create a response object.  The response object can be extended with
  * header information and then be used any number of times.
  *
- * @param size size of the data portion of the response, MHD_SIZE_UNKNOW for unknown
+ * @param size size of the data portion of the response, MHD_SIZE_UNKNOWN for unknown
  * @param block_size preferred block size for querying crc (advisory only,
  *                   MHD may still call crc using smaller chunks); this
  *                   is essentially the buffer size used for IO, clients
@@ -1106,6 +1082,18 @@ struct MHD_Response *MHD_create_response_from_data (size_t size,
                                                     int must_free,
                                                     int must_copy);
 
+
+/**
+ * Create a response object.  The response object can be extended with
+ * header information and then be used any number of times.
+ *
+ * @param size size of the data portion of the response
+ * @param fd file descriptor referring to a file on disk with the data; will be closed when response is destroyed
+ * @return NULL on error (i.e. invalid arguments, out of memory)
+ */
+struct MHD_Response *MHD_create_response_from_fd (size_t size,
+						  int fd);
+
 /**
  * Destroy a response object and associated resources.  Note that
  * libmicrohttpd may keep some of the resources around if the response
@@ -1230,14 +1218,27 @@ int MHD_destroy_post_processor (struct MHD_PostProcessor *pp);
 
 /* ********************** generic query functions ********************** */
 
-
 /**
  * Information about a connection.
  */
 union MHD_ConnectionInfo
 {
-  enum MHD_GNUTLS_CipherAlgorithm cipher_algorithm;
-  enum MHD_GNUTLS_Protocol protocol;
+
+  /**
+   * Cipher algorithm used, of type "enum gnutls_cipher_algorithm".
+   */
+  int /* enum gnutls_cipher_algorithm */ cipher_algorithm;
+
+  /**
+   * Protocol used, of type "enum gnutls_protocol".
+   */
+  int /* enum gnutls_protocol */ protocol;
+
+  /**
+   * GNUtls session handle, of type "gnutls_session_t".
+   */
+  void * /* gnutls_session_t */ tls_session;
+
   /**
    * Address information for the client.
    */
