@@ -19,7 +19,7 @@
  */
 
 /**
- * @file mhds_get_test.c
+ * @file tls_authentication_test.c
  * @brief  Testcase for libmicrohttpd HTTPS GET operations
  * @author Sagie Amir
  */
@@ -40,7 +40,7 @@ extern const char ca_cert_pem[];
 extern const char srv_signed_cert_pem[];
 extern const char srv_signed_key_pem[];
 
-const char *ca_cert_file_name = "ca_cert_pem";
+const char *ca_cert_file_name = "tmp_ca_cert.pem";
 
 /*
  * test HTTPS transfer
@@ -108,11 +108,12 @@ test_daemon_get (FILE * test_fd, char *cipher_suite, int proto_version)
   /* TODO merge into send_curl_req */
   curl_easy_setopt (c, CURLOPT_SSL_VERIFYPEER, 1);
   curl_easy_setopt (c, CURLOPT_CAINFO, ca_cert_file_name);
-
   curl_easy_setopt (c, CURLOPT_SSL_VERIFYHOST, 0);
 
   curl_easy_setopt (c, CURLOPT_FAILONERROR, 1);
 
+  curl_easy_setopt (c, CURLOPT_FAILONERROR, 1);
+
   /* NOTE: use of CONNECTTIMEOUT without also
      setting NOSIGNAL results in really weird
      crashes on my system! */
@@ -173,7 +174,7 @@ setup_ca_cert ()
 {
   FILE *cert_fd;
 
-  if (NULL == (cert_fd = fopen (ca_cert_file_name, "w+")))
+  if (NULL == (cert_fd = fopen (ca_cert_file_name, "wb+")))
     {
       fprintf (stderr, "Error: failed to open `%s': %s\n",
                ca_cert_file_name, strerror (errno));
@@ -204,11 +205,6 @@ main (int argc, char *const *argv)
   FILE *test_fd;
   unsigned int errorCount = 0;
 
-  if (curl_check_version (MHD_REQ_CURL_VERSION))
-    {
-      return -1;
-    }
-
   if ((test_fd = setup_test_file ()) == NULL || setup_ca_cert () == NULL)
     {
       fprintf (stderr, MHD_E_TEST_FILE_CREAT);
